{% extends "!indexsidebar.html" %}


<h3 style="background:gray;border: solid 1px black;margin:0.2em;padding:0.2em ">Author: Thomas Cokelaer</h3>
<h3 style="background:gray;border: solid 1px black;margin:0.2em;padding:0.2em ">Download</h3>
<p>Current version: <b>{{ version }}</b></p>
<p> Get Spectrum <a href="tobedone">Pypi coming soon</a></p>


<h3 style="background:gray;border: solid 1px black;margin:0.2em;padding:0.2em ">Questions? Suggestions?</h3>
<a href="mailto:cno-contact@ebi.ac.uk">cno-contact</a> 


